@@ -1,5 +1,5 @@
 /***************************************************************************
-                          draganddrop.cpp  -  description
+                    draganddrop.cpp  -  Drag & Drop handler
                              -------------------
     begin                : Thu Aug 28 2003
     copyright            : (C) 2003 by Gabor Torok
@@ -15,6 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "../common/constants.h"
 #include "draganddrop.h"
 
 /**
