@@ -1,5 +1,5 @@
 /***************************************************************************
-                          widgetview.cpp  -  description
+                     widgetview.cpp  -  Widget view class
                              -------------------
     begin                : Sat May 3 2003
     copyright            : (C) 2003 by Gabor Torok
@@ -15,6 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "../common/constants.h"
 #include "widgetview.h"
 
 /**
