@@ -1,5 +1,5 @@
 /***************************************************************************
-                          widgetview.h  -  description
+                      widgetview.h  -  Widget view class
                              -------------------
     begin                : Sat May 3 2003
     copyright            : (C) 2003 by Gabor Torok
@@ -17,6 +17,7 @@
 
 #ifndef WIDGETVIEW_H
 #define WIDGETVIEW_H
+#pragma once
 
 #include "gui.h"
 #include "widget.h"
@@ -25,13 +26,14 @@
   *@author Gabor Torok
   */
 
+/// This class lets you draw a widget.
 class WidgetView {
- public: 
-  WidgetView();
-  virtual ~WidgetView();
-  
-  /** Draw the widget. */
-  virtual void drawWidgetContents(Widget *w) = 0;
+public:
+	WidgetView();
+	virtual ~WidgetView();
+
+	/** Draw the widget. */
+	virtual void drawWidgetContents( Widget *w ) = 0;
 };
 
 #endif
