@@ -1,8 +1,8 @@
 /***************************************************************************
-                          quickhull.h  -  description
+                    quickhull.h  -  Quick Hull Algorithm
                              -------------------
     begin                : Sun July 8 2007
-    copyright            : (C) 2003 by Gabor Torok
+    copyright            : (C) 2007 by Gabor Torok
     email                : cctorok@yahoo.com
 ***************************************************************************/
 
@@ -20,11 +20,12 @@ http://www.cse.yorku.ca/~aaw/Hang/quick_hull/Algorithm.html
 */
 #ifndef QUICK_HULL_H
 #define QUICK_HULL_H
+#pragma once
 
 #include <vector>
 #include <set>
-#include "common/constants.h"
 
+/// Quick Hull algorithm.
 class QuickHull {
 public:
 	static void findConvexHull( std::vector<CVector2*> *in, std::vector<CVector2*> *out );
