@@ -1,6 +1,6 @@
 
 /***************************************************************************
-                          location.cpp  -  description
+                    location.cpp  -  Location on a level map
                              -------------------
     begin                : Mon May 12 2003
     copyright            : (C) 2003 by Gabor Torok
@@ -16,6 +16,7 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "../common/constants.h"
 #include "location.h"
 
 
