@@ -1,5 +1,5 @@
 /***************************************************************************
-                          mapadapter.cpp  -  description
+               mapadapter.cpp  -  Map related utility interface
                              -------------------
     begin                : Sat May 3 2003
     copyright            : (C) 2003 by Gabor Torok
@@ -15,6 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "../common/constants.h"
 #include "mapadapter.h"
 
 MapAdapter::MapAdapter() {
