@@ -1,5 +1,5 @@
 /***************************************************************************
-                          render.h  -  description
+                   render.h  -  Rendering interface class
                              -------------------
     begin                : Sat May 3 2003
     copyright            : (C) 2003 by Gabor Torok
@@ -17,21 +17,21 @@
 
 #ifndef RENDER_H
 #define RENDER_H
+#pragma once
 
-// definitely outside of dir                    
-#include "../common/constants.h"
+// definitely outside of dir
 #include "../persist.h"
 #include "../preferences.h"
 #include "../util.h"
 
-/** 
+/**
  *@author Gabor Torok
- * 
+ *
  * The class thru which this directory talks to the rest of the code.
  * Never include this file outside this dir. (use renderlib.h instead.)
  */
 class Render {
-  public:
+public:
 };
 
 #endif
