@@ -1,5 +1,5 @@
 /***************************************************************************
-                          sqbinding.h  -  description
+                   sqbinding.h  -  Squirrel binding - Console
                              -------------------
     begin                : Sat Oct 8 2005
     copyright            : (C) 2005 by Gabor Torok
@@ -17,23 +17,23 @@
 
 #ifndef CONSOLE_PRINTER_H
 #define CONSOLE_PRINTER_H
+#pragma once
 
 #include <iostream>
 #include <string>
 #include <vector>
 
-/**
- * An interface for console printing.
- */
+/// An interface for console printing.
+
 class ConsolePrinter {
 public:
-  ConsolePrinter() {
-  }
+	ConsolePrinter() {
+	}
 
-  virtual ~ConsolePrinter() {
-  }
+	virtual ~ConsolePrinter() {
+	}
 
-  virtual void printToConsole( const char *s ) = 0;
+	virtual void printToConsole( const char *s ) = 0;
 };
 
 #endif
