@@ -8,7 +8,7 @@ struct Creature
 {
 	std::string name, portrait;
 	std::string md2, skin, level, hp, mp, armor, rareness, speed, scale, npc,npcStartX,npcStartY;
-	std::vector <std::string> inventory;
+	std::vector <std::string> backpack;
 	std::vector <std::string> spells;
 	std::map <std::string, std::string> skills;
 	// portrait is optional
