@@ -1,5 +1,5 @@
 /***************************************************************************
-                          upload.h  -  description
+                upload.h  -  Uploads final game score to web
                              -------------------
     begin                : Sat May 3 2003
     copyright            : (C) 2003 by Gabor Torok
@@ -17,10 +17,14 @@
 
 #ifndef UPLOAD
 #define UPLOAD
+#pragma once
 
+/// Uploads the death score and description to the web.
 class Upload {
 public:
-	static int uploadScoreToWeb( char *score, char *result );
+	enum { RESULT_SIZE = 300 };
+	typedef char RESULT[ RESULT_SIZE ];
+	static int uploadScoreToWeb( char *score, RESULT& result );
 };
 
 #endif
